function detection(handles)
%DETECTION Summary of this function goes here
%   Detailed explanation goes here

path = get(handles.Folder_text, 'String');
files = get(handles.Files_listbox, 'String');
nRuns = size(files, 1);

if isequal(nRuns, 0)
    errordlg('Select data files', 'Error', 'modal');
    return;
end

parameters = get(handles.Parameters_table, 'data');
alpha = parameters(1);
beta = parameters(2);
rMin = parameters(3);
rMax = parameters(4);
nR = parameters(5);

A = log((1-beta)/alpha);
B = log((beta)/(1 - alpha));

if rMax == rMin
    rDelta = 0;
    nR = 1;
else
    rDelta = (rMax - rMin) / (nR - 1);
end

P0 = eye(4, 4);

for k = 1:nRuns
    load([path char(files(k, 1))], 'Trajectory', 'x0', 'tau', 'Gd', 'Q', 'X', 'H', 'R', 'Z');
    
    cla(handles.OXY_plot)
    OXY_limits = [(x0(1) - 0.01) (x0(1) + 0.01) (x0(3) - 0.01) (x0(3) + 0.01)];
    axis(handles.OXY_plot, OXY_limits);
    
    set(handles.Files_listbox, 'value', k);
    set(handles.Trajectory_table, 'data', Trajectory);
    set(handles.X0_table, 'data', x0');
    set(handles.Q_table, 'data', Q);
    set(handles.R_table, 'data', R);
    set(handles.Tau_edit, 'string', tau);
    
    n = sum(cell2mat(Trajectory(:, 2)));
    
    adaptEstX = zeros(4, n);
    isOn = zeros(2 * nR + 1, n);
    lambda = zeros(2 * nR + 1, n);
    active = zeros(1, n);
    detectedTrajectory = Trajectory;
    
    nOn = 0;
    activeIndex = 1; % 1 == Straight or set to some initialIndex;
    P = P0;
    prevEstXi = x0; % Last estimates of the previous section
    
    t = 1;
    prevFaultMoment = 1;
    
    for m = 1:size(Trajectory, 1)
        setFilters(t);
        nextFaultMoment = t + Trajectory{m, 2};
        isFaultDetected = false;
        
        while ((t < nextFaultMoment) && not(isFaultDetected))
            active(t) = activeIndex;
            makeStep(t);
            calculateLambda(t);
            makeDecision(t);
            
            t = t + 1;
        end
        
        if (isFaultDetected)
            % Second pass with stabilazied P and initial estimates from the
            % previous section
            FB(activeIndex).estXi = prevEstXi;
            
            switch getFilterMode(activeIndex, nR)
                case 1 % Straight
                    FB(activeIndex).F = matrixFPRD(tau);
                    FB(activeIndex).Bd = matrixBdPRD();
                case 2 % Left
                    FB(activeIndex).F = matrixFRDO(FB(activeIndex).estXi, FB(activeIndex).r, tau);
                    FB(activeIndex).Bd = matrixBdRDOL(FB(activeIndex).estXi, FB(activeIndex).r, tau);
                case 3 % Right
                    FB(activeIndex).F = matrixFRDO(FB(activeIndex).estXi, FB(activeIndex).r, tau);
                    FB(activeIndex).Bd = matrixBdRDOR(FB(activeIndex).estXi, FB(activeIndex).r, tau);
            end
            
            for i = prevFaultMoment : t - 1
                active(i) = activeIndex;
                isOn(activeIndex, i) = 1;
                [FB(activeIndex).estXi, FB(activeIndex).P, FB(activeIndex).nu, FB(activeIndex).Sigma] = estStepKalman(FB(activeIndex).estXi, Z(:,i), FB(activeIndex).P, FB(activeIndex).F, FB(activeIndex).Bd, Gd, Q, H, R);
                adaptEstX(:, i) = FB(activeIndex).estXi;
            end
        end
        
        for i = t : nextFaultMoment - 1
            active(i) = activeIndex;
            isOn(activeIndex, i) = 1;
            [FB(activeIndex).estXi, FB(activeIndex).P, FB(activeIndex).nu, FB(activeIndex).Sigma, K] = estStepKalman(FB(activeIndex).estXi, Z(:,i), FB(activeIndex).P, FB(activeIndex).F, FB(activeIndex).Bd, Gd, Q, H, R);
            adaptEstX(:, i) = FB(activeIndex).estXi;
        end
        
        prevEstXi = FB(activeIndex).estXi;
        P = P0;
        t = nextFaultMoment;
        prevFaultMoment = nextFaultMoment;
        
        switch getFilterMode(activeIndex, nR)
            case 1
                detectedTrajectory{m, 1} = 'S';
                detectedTrajectory{m, 3} = [];
            case 2
                detectedTrajectory{m, 1} = 'L';
                detectedTrajectory{m, 3} = FB(activeIndex).r;
            case 3
                detectedTrajectory{m, 1} = 'R';
                detectedTrajectory{m, 3} = FB(activeIndex).r;
        end
    end
    
    detPlot(k, X, Z, adaptEstX, handles);
    
    save([path char(files(k, 1))], '-append', 'detectedTrajectory', 'adaptEstX', 'active', 'isOn', 'lambda', 'alpha', 'beta', 'rMin', 'rMax', 'nR');
    pause(0.01);
end

    function setFilters(t)
        for f = 1:2*nR+1
            FB(f).estXi = prevEstXi;
            FB(f).P = P;
            FB(f).isOn = 1;
            FB(f).lambda = 0;
            
            isOn(f, t) = 1;
            lambda(f, t) = 0;
        end
        
        nOn = 2 * nR + 1;
        
        % Straight
        FB(1).F = matrixFPRD(tau);
        FB(1).Bd = matrixBdPRD();
        
        % Left
        for f = 2:nR+1
            FB(f).r = rMin + (f - 2) * rDelta;
            FB(f).F = matrixFRDO(prevEstXi, FB(f).r, tau);
            FB(f).Bd = matrixBdRDOL(prevEstXi, FB(f).r, tau);
        end
        
        % Right
        for  f = nR+2:2*nR+1
            FB(f).r = rMin + (f - nR - 2) * rDelta;
            FB(f).F = matrixFRDO(prevEstXi, FB(f).r, tau);
            FB(f).Bd = matrixBdRDOR(prevEstXi, FB(f).r, tau);
        end
    end


    function makeStep(t)
        for f = 1:2*nR+1
            if FB(f).isOn == 1
                [FB(f).estXi, FB(f).P, FB(f).nu, FB(f).Sigma] = estStepKalman(FB(f).estXi, Z(:,t), FB(f).P, FB(f).F, FB(f).Bd, Gd, Q, H, R);
                isOn(f, t) = 1;
            end
        end
        
        adaptEstX(:, t) = FB(activeIndex).estXi;
    end


    function calculateLambda(t)
        for f = 1:2*nR+1
            if (FB(f).isOn == 1) && not(activeIndex == f)
                FB(f).lambda = FB(f).lambda + 0.5 * (log(det(FB(activeIndex).Sigma)) - log(det(FB(f).Sigma)) + FB(activeIndex).nu' / FB(activeIndex).Sigma * FB(activeIndex).nu - FB(f).nu' / FB(f).Sigma * FB(f).nu);
                lambda(f, t) = FB(f).lambda;
            end
        end
    end


    function makeDecision(t)
        lambdaMax = B;
        maxIndex = 0;
        
        for f = 1:2*nR+1
            if (FB(f).isOn == 1) && not(activeIndex == f)
                if (FB(f).lambda <= B) || (FB(f).lambda >= A)
                    FB(f).isOn = 0;
                    nOn = nOn - 1;
                end
                
                if (FB(f).lambda >= A) && (FB(f).lambda > lambdaMax)
                    lambdaMax = FB(f).lambda;
                    maxIndex = f;
                end
            end
        end
        
        if (maxIndex > 0)
            activeIndex = maxIndex;
            isFaultDetected = true;
            return;
        end
        
        if (nOn == 1)
            isFaultDetected = true;
            return;
        end
        
        isFaultDetected = false;
        return;
    end

end