function estimation(handles)
%ESTIMATION Summary of this function goes here
%   Detailed explanation goes here

path = get(handles.Folder_text, 'string');
files = get(handles.Files_listbox, 'string');
nRuns = size(files, 1);

if isequal(nRuns, 0)
    errordlg('Select data files', 'Error', 'modal');
    return;
end

algIndex = get(handles.Algorithm_listbox, 'value');
list = get(handles.Algorithm_listbox, 'string');
filter = list{algIndex};

P0 = eye(4, 4);
S_P0 = eye(4,4);
U_P0 = eye(4,4);
D_P0 = eye(4,4);

for i = 1:nRuns
    load([path char(files(i, 1))], 'Trajectory', 'x0', 'tau', 'Gd', 'Q', 'X', 'H', 'R', 'Z');
    
    cla(handles.OXY_plot)
    
    set(handles.Files_listbox, 'value', i);
    set(handles.Trajectory_table, 'data', Trajectory);
    set(handles.X0_table, 'data', x0');
    set(handles.Q_table, 'data', Q);
    set(handles.R_table, 'data', R);
    set(handles.Tau_edit, 'string', tau);
    
    optEstX = zeros(4, size(X, 2));
    oldEstXi = x0;  % Last estimates
    prevEstXi = x0; % Last estimates of the previous section
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if nnz(Q) ~= 0
        S_Q = chol(Q);
        [U_Q, D_Q] = ud(Q);
    else
        S_Q = zeros(2, 2);
        U_Q = eye(2, 2);
        D_Q = zeros(2,2);
    end
    
    if nnz(R) ~= 0
        S_R = chol(R);
        [U_R, D_R] = ud(R);
    else
        S_R = zeros(2, 2);
        U_R = eye(2, 2);
        D_R = zeros(2,2);
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    nSections = size(Trajectory,1);
    
    t0 = 0;
    for m = 1:nSections
        type = upper(Trajectory{m,1});
        length = Trajectory{m,2};
        
        if type == 'S'
            F = matrixFPRD(tau);
            Bd = matrixBdPRD();
        else
            r = Trajectory{m,3};
            F = matrixFRDO(oldEstXi, r, tau);
            
            if type == 'L'
                Bd = matrixBdRDOL(oldEstXi, r, tau);
            end
            
            if type == 'R'
                Bd = matrixBdRDOR(oldEstXi, r, tau);
            end
        end
        
        P = P0;
        S_P = S_P0;
        U_P = U_P0;
        D_P = D_P0;
        
        for t = t0 + 1 : t0 + length
            if strcmp(filter, 'CKF')
                [optEstX(:, t), P, nu, Sigma, K] = estStepKalman(oldEstXi, Z(:, t), P, F, Bd, Gd, Q, H, R);
            elseif strcmp(filter, 'SRCF')
                [optEstX(:, t), S_P] = estStepSRCF(oldEstXi, Z(:, t), S_P, F, Bd, Gd, S_Q, H, S_R);
            elseif strcmp(filter, 'UD-CF')
                [optEstX(:, t), U_P, D_P] = estStepUD(oldEstXi, Z(:, t), U_P, D_P, F, Bd, Gd, U_Q, D_Q, H, U_R, D_R);
            end
            
            oldEstXi = optEstX(:, t);
        end
        
        % Second pass with stabilazied P and initial estimates from the
        % previous section
        oldEstXi = prevEstXi;
        for t = t0 + 1 : t0 + length
            if strcmp(filter, 'CKF')
                [optEstX(:, t), P, nu, Sigma, K] = estStepKalman(oldEstXi, Z(:, t), P, F, Bd, Gd, Q, H, R);
            elseif strcmp(filter, 'SRCF')
                [optEstX(:, t), S_P] = estStepSRCF(oldEstXi, Z(:, t), S_P, F, Bd, Gd, S_Q, H, S_R);
            elseif strcmp(filter, 'UD-CF')
                [optEstX(:, t), U_P, D_P] = estStepUD(oldEstXi, Z(:, t), U_P, D_P, F, Bd, Gd, U_Q, D_Q, H, U_R, D_R);
            end
            
            oldEstXi = optEstX(:, t);
        end
        
        prevEstXi = optEstX(:, t);
        t0 = t;
    end
    
    estPlot(i, X, Z, optEstX, handles);
    
    optFilter = filter;
    save([path char(files(i, 1))], '-append', 'optEstX', 'optFilter');
    
    pause(0.1);
end

end
