function measurements(handles)
%MEASUREMENTS Summary of this function goes here
%   Detailed explanation goes here

path = get(handles.Folder_text, 'string');
files = get(handles.Files_listbox, 'string');
nRuns = size(files, 1);

if isequal(nRuns, 0)
    errordlg('Select data files', 'Error', 'modal');
    return;
end

rng(round(str2double(get(handles.Seed_edit, 'string'))), 'twister');

index = get(handles.Observable_listbox, 'value');
switch index
    case 1
        H = [1 0 0 0;
            0 0 1 0];
    case 2
        H = eye(4);
end

R = cell2mat(get(handles.R_table, 'data'));

if nnz(R) ~= 0
    S_R = chol(R);
else
    S_R = R;
end


for i = 1:nRuns
    load([path char(files(i, 1))], 'Trajectory', 'x0', 'tau', 'Gd', 'Q', 'X');
    
    cla(handles.OXY_plot)
    
    set(handles.Files_listbox, 'value', i);
    set(handles.Trajectory_table, 'data', Trajectory);
    set(handles.X0_table, 'data', x0');
    set(handles.Q_table, 'data', Q);
    set(handles.Tau_edit, 'string', tau);
    
    nSections = size(Trajectory,1);
    n = sum(cell2mat(Trajectory(:, 2)));
    Z = zeros(size(S_R, 1), n);
    
    t0 = 0;
    for m = 1:nSections
        length = Trajectory{m,2};
        
        for t = t0 + 1 : t0 + length
            v = randn(size(S_R, 1), 1);
            Z(:, t) = H * X(:, t) + S_R * v;
        end
        
        t0 = t;
    end
    
    measPlot(i, X, Z, handles);
    
    save([path char(files(i, 1))], '-append', 'H', 'R', 'Z');
    
    pause(0.1);
end


end

