function modeling(handles)
%MODELING Summary of this function goes here
%   Detailed explanation goes here

nRuns = round(str2double(get(handles.nRuns_edit, 'string')));
path = [get(handles.Output_folder_text, 'string') '\'];
fname = get(handles.Name_edit, 'string');

x0 = get(handles.X0_table, 'data')'; %   , x,vx,y,vy
tau = str2double(get(handles.tau_edit, 'string'));
Q = get(handles.Q_table, 'data');
Trajectory = get(handles.Trajectory_table,'data');
nSections = size(Trajectory,1);

rng(round(str2double(get(handles.Seed_edit, 'string'))), 'twister');

Gd = [0 0;
    1 0;
    0 0;
    0 1];

if nnz(Q) ~= 0
    S_Q = chol(Q);
else
    S_Q = zeros(2, 2);
end

n = sum(cell2mat(Trajectory(:, 2)));
X = zeros(4, n);

for i = 1:nRuns
    if get(handles.Redraw_checkbox, 'Value') == 1
        cla(handles.OXY_plot);
        cla(handles.X_plot);
        cla(handles.Vx_plot);
        cla(handles.Y_plot);
        cla(handles.Vy_plot);
    end

    t0 = 0;
    oldXi = x0;
    
    for m = 1:nSections
        type = upper(Trajectory{m,1});
        length = Trajectory{m,2};
        
        if type == 'S'
            F = matrixFPRD(tau);
            Bd = matrixBdPRD();
        else
            r = Trajectory{m,3};
            F = matrixFRDO(oldXi, r, tau);
            
            if type == 'L'
                Bd = matrixBdRDOL(oldXi, r, tau);
            end
            
            if type == 'R'
                Bd = matrixBdRDOR(oldXi, r, tau);
            end
        end
        
        for t = t0 + 1 : t0 + length
            X(:, t) = modStep(oldXi, F, Bd, Gd, S_Q);
            oldXi = X(:, t);
        end
        
        t0 = t;
    end
    
    modPlot(i, X, handles);
    save([path fname '_' num2str(i) '.mat'], 'Trajectory', 'x0', 'tau', 'Gd', 'Q', 'X');
    pause(0.1);
end









Scheme = zeros(nSections, 3);
for m = 1:nSections
    Scheme(m, 2) = Trajectory{m, 2};
    type = upper(Trajectory{m, 1});
    
    if type == 'S'
        Scheme(m, 1) = 0;
    end
    
    if type == 'L'
        Scheme(m, 1) = -1;
        Scheme(m, 3) = Trajectory{m, 3};
    end
    
    if type == 'R'
        Scheme(m, 1) = 1;
        Scheme(m, 3) = Trajectory{m, 3};
    end
end


% save([path fname '_Scheme.txt'], 'Scheme', '-ascii', '-double');
% save([path fname '_x0.txt'], 'x0', '-ascii', '-double');
% save([path fname '_tau.txt'], 'tau', '-ascii', '-double');
% save([path fname '_Gd.txt'], 'Gd', '-ascii', '-double');
% save([path fname '_Q.txt'], 'Q', '-ascii', '-double');


