function plots()
%PLOTS Summary of this function goes here
%   Detailed explanation goes here

[file,path] = uigetfile('*.mat');
if not(isequal(file,0))
    disp(['Selected file: ', fullfile(path,file)]);
    
    load([path file], 'Trajectory', 'x0', 'tau', 'Gd', 'Q', 'X', 'H', 'R', 'Z', 'optEstX', 'detectedTrajectory', 'adaptEstX', 'active', 'isOn', 'lambda', 'alpha', 'beta', 'rMin', 'rMax', 'nR');
    
    if rMax == rMin
        nR = 1;
    end
    
    X = [x0 X];
    optEstX = [x0 optEstX];
    adaptEstX = [x0 adaptEstX];
    
    switch size(Z, 1)
        case 2
            Z = [[x0(1); x0(3)] Z];
        case 4
            Z = [x0' Z];
    end
    
    % Trajectory, measurements
    figure
    
    plot(Z(1,:), Z(2,:), '.', 'color', [0,0,0]+0.5)
    hold on
    
    disp('Trajectory:');
    nSections = size(Trajectory,1);
    t = 1;
    for m = 1:nSections
        type = upper(Trajectory{m,1});
        length = Trajectory{m,2};
        
        if type ~= 'S'
            radius = num2str(Trajectory{m,3});
        else
            radius = '';
        end
        
        disp([type ' ' num2str(length) ' ' radius]);
        
        switch type
            case 'S'
                color = 'r';
            case 'L'
                color = 'b';
            case 'R'
                color = 'g';
        end
        
        % plot section of X on Oxy
        plot(X(1, t: t + length), X(3, t: t + length), color, 'LineWidth', 4);
        hold on
        
        t = t + length;
    end
    
    grid on
    
    % Optimal estimation, measurements
    figure
    
    plot(Z(1,:), Z(2,:), '.', 'color', [0,0,0]+0.5)
    hold on
    
    plot(optEstX(1,:), optEstX(3,:), 'm', 'LineWidth', 2)
    grid on
    
    % Adaptive estimation, measurements
    figure
    
    plot(Z(1,:), Z(2,:), '.', 'color', [0,0,0]+0.5)
    hold on
    
    disp('Detected trajectory:');
    nSections = size(detectedTrajectory,1);
    t = 1;
    for m = 1:nSections
        type = upper(detectedTrajectory{m,1});
        length = detectedTrajectory{m,2};
        
        if type ~= 'S'
            radius = num2str(detectedTrajectory{m,3});
        else
            radius = '';
        end
        
        disp([type ' ' num2str(length) ' ' radius]);
        
        switch type
            case 'S'
                color = 'r';
            case 'L'
                color = 'b';
            case 'R'
                color = 'g';
        end
        
        % plot section of X on Oxy
        plot(adaptEstX(1, t: t + length), adaptEstX(3, t: t + length), color, 'LineWidth', 2);
        hold on
        
        t = t + length;
    end
    
    grid on
    
    m = size(isOn, 1);
    n = size(isOn, 2);
    
    % Active filter
    figure
    plot(active)
    axis([1 n 1 m]);
    grid on
    
    % Lambda
    figure
    
    A = log((1-beta)/alpha);
    B = log((beta)/(1 - alpha));
    
    plot(1:n, A * ones(1, n), 'k', 'LineWidth', 1);
    text(2, A + 0.5, ['A = ' num2str(A)]);
    hold on
    
    plot(1:n, B * ones(1, n), 'k', 'LineWidth', 1);
    text(2, B - 0.5, ['B = ' num2str(B)]);
    hold on
    
    for i = 1:m
        switch getFilterMode(i, nR)
            case 1
                color = 'r';
            case 2
                color = 'b';
            case 3
                color = 'g';
        end
        
        plot(lambda(i, :), color);
        hold on
    end
    
    axis([1 n floor(B-1) ceil(A+1)]);
    grid on
else
    disp('User selected Cancel');
end

end